#!/usr/bin/python
"""
People - Yet Another Contact manager
    A VCard-based contact manager.

Copyright (C) 2001 James ''Wez''Weatherall

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""

try: import pylau
except ImportError: pass

import gtk
import sys
import os
import time

from vobject import VCard
from vobject import VLowLevel

def MenuItem(title, func, data=None, name=None):
    i=gtk.GtkMenuItem(title)
    if func: i.connect("activate", func)
    if name: i.set_name(name)
    if data: i.set_data(data[0], data[1])
    i.show()
    return i

def Menu(items):
    m=gtk.GtkMenu()
    for i in items:
        m.add(i)
    return m

def HButtonBox(*buttons):
    box=gtk.GtkHButtonBox()
    box.set_spacing(0)
    box.set_child_ipadding_default(0, 0)
    box.set_child_size_default(0,0)
    for i in buttons:
        box.add(i)
    return box

def Button(title, func, name=None):
    b=gtk.GtkButton(title)
    if name: b.set_name(name)
    b.connect("clicked", func)
    b.show()
    return b

# - VCard manager class
#   The VCard class is defined in VCard module
def _cmp_cards(a, b):
    if hasattr(a, "_cached_name"): an=a._cached_name
    else: a._cached_name=an=str(a.get_name())
    if hasattr(b, "_cached_name"): bn=b._cached_name
    else: b._cached_name=bn=str(b.get_name())
    if an < bn: return -1
    elif an > bn: return 1
    return 0

class VCardMgr:
    def __init__(self, filename):
        self.filename = filename
        try: self.objects = VLowLevel.VReadObjects(open(filename))
        except:
            print "warning: reading old temporary database"
            try: self.objects = VLowLevel.VReadObjects(open(filename)+".tmp")
            except: self.objects = []
        self.timer=None
        self.modified = 0
        self.last_dump=time.time()
    def get_vcards(self):
        return filter(lambda x:isinstance(x, VCard.VCard), self.objects)
    def get_sorted_vcards(self):
        cards=self.get_vcards()
        cards.sort(_cmp_cards)
        return cards
    def add_vcard(self, card):
        if card not in self.objects:
            self.objects.append(card)
            self.modified=1
    def rem_vcard(self, card):
        if card in self.objects:
            self.objects.remove(card)
            self.modified=1
    def new_vcard(self):
        return VCard.VCard()
    def dump_now(self):
        for object in self.get_vcards():
            if time.mktime(object.get_revision()) > self.last_dump:
                self.modified = 1
        if not self.modified: return 0
        print "people: committing database"
        try:
            # - Write to a new file
            f=open(self.filename+".tmp", "w")
            for object in self.objects:
                f.write(repr(object))
            f.close()
            print "people:wrote new database"
            if os.path.exists(self.filename): os.remove(self.filename)
            try:
                os.rename(self.filename+".tmp", self.filename)
                self.last_dump=time.time()
                self.modified=0
                return 1
            except:
                print "warning: failed to rename temporary database"
        except:
            print "error: unable to commit changes"
        return 0
    def dump(self):
        if not self.timer:
            # - Start the commit timer
            self.timer = gtk.timeout_add(30000, self.on_dump_timeout)
    def on_dump_timeout(self):
        done = self.dump_now()
        if not done: self.timer = None
        return done

# - Views
def _LabelledEntry(label, defval, box):
    frame = gtk.GtkFrame(label)
    entry = gtk.GtkEntry()
    entry.set_text(defval)
    frame.add(entry)
    box.pack_start(frame, 0, 0)
    return entry

class _VCard_Pane:
    def __init__(self, vcard_view, title):
        self.vcard_view=vcard_view
        self.vcard = vcard_view.vcard
        self.vcard_view.book.append_page(self.gui, gtk.GtkLabel(title))

class _VCard_ID(_VCard_Pane):
    def __init__(self, vcard_view):
        self.gui = gtk.GtkVBox()
        _VCard_Pane.__init__(self, vcard_view, "I.D.")
        name=self.vcard.get_name()
        self.tags={}
        for tag in ["title", "first", "middle", "last"]:
            self.tags[tag]=_LabelledEntry(tag.capitalize(), getattr(name,tag), self.gui)
    def commit(self):
        changed = 0
        name=self.vcard.get_name()
        for tag in self.tags.keys():
            if self.tags[tag].get_text() != getattr(name, tag):
                setattr(name, tag, self.tags[tag].get_text())
                changed = 1
        if changed:   
            self.vcard.set_name(name)
            self.vcard.set_revision()

class _VCard_Photo(_VCard_Pane):
    def __init__(self, vcard_view):
        self.gui = gtk.GtkVBox()
        _VCard_Pane.__init__(self, vcard_view, "Photo")
        # *** hacking VCard API...
        photo=self.vcard.get_photo_jpeg_url()
        self.photo=_LabelledEntry("Photo URL", photo, self.gui)
    def commit(self):
        photo=self.vcard._VCard__get_prop("PHOTO", "")
        if self.photo.get_text() != photo:
            self.vcard.set_photo_jpeg_url(self.photo.get_text())
            self.vcard.set_revision()

class _VCard_Mobile(_VCard_Pane):
    def __init__(self, vcard_view):
        self.gui = gtk.GtkVBox()
        _VCard_Pane.__init__(self, vcard_view, "Mobile")
        number = self.vcard.get_mobile_phone()
        self.mobile=_LabelledEntry("Telephone", number.number, self.gui)
    def commit(self):
        number = self.vcard.get_mobile_phone()
        if self.mobile.get_text() != number.number:
            number.number = self.mobile.get_text()
            self.vcard.set_mobile_phone(number)
            self.vcard.set_revision()

class _VCard_Email(_VCard_Pane):
    def __init__(self, vcard_view):
        self.gui = gtk.GtkVBox()
        _VCard_Pane.__init__(self, vcard_view, "Email")
        emails=self.vcard.get_email_addrs()
        self.emails=[]
        for i in range(4):
            if i < len(emails): value=emails[i].address
            else: value=""
            self.emails.append(_LabelledEntry("Email "+str(i), value, self.gui))
    def commit(self):
        changed = 0
        emails=self.vcard.get_email_addrs()
        new_emails=[]
        for i in range(len(self.emails)):
            item = VCard.Email()
            item.address = self.emails[i].get_text()
            if i < len(emails): value=emails[i]
            else: value = ""
            if value != item.address: changed = 1
            if item.address: new_emails.append(item)
        if changed:
            self.vcard.set_email_addrs(new_emails)
            self.vcard.set_revision()

class _VCard_Addr(_VCard_Pane):
    def __init__(self, vcard_view, home=1):
        self.gui=gtk.GtkScrolledWindow()
        if home: name = "Home"
        else: name = "Work"
        _VCard_Pane.__init__(self, vcard_view, name)
        self.home=home

        # - Addr
        addr=self.get_addr()
        self.addr_tags={}
        vbox=gtk.GtkVBox()
        self.gui.add_with_viewport(vbox)
        for tag in ["extended", "street", "city", "region", "postcode", "country"]:
            self.addr_tags[tag]=_LabelledEntry(tag.capitalize(), getattr(addr,tag), vbox)

        # - Telephone
        tel=self.get_telephone()
        self.tel_number=_LabelledEntry("Telephone", tel.number, vbox)        
    def get_addr(self):
        if self.home: addr=self.vcard.get_home_address()
        else: addr=self.vcard.get_work_address()
        return addr
    def set_addr(self, addr):
        if self.home: addr=self.vcard.set_home_address(addr)
        else: addr=self.vcard.set_work_address(addr)
    def get_telephone(self):
        if self.home: addr=self.vcard.get_home_phone()
        else: addr=self.vcard.get_work_phone()
        return addr
    def set_telephone(self, addr):
        if self.home: addr=self.vcard.set_home_phone(addr)
        else: addr=self.vcard.set_work_phone(addr)
    def commit(self):
        changed = 0
        addr=self.get_addr()
        for tag in self.addr_tags.keys():
            if self.addr_tags[tag].get_text() != getattr(addr, tag):
                setattr(addr, tag, self.addr_tags[tag].get_text())
                changed = 1
        tel=self.get_telephone()
        if tel.number != self.tel_number.get_text():
            tel.number = self.tel_number.get_text()
            changed = 1
        if changed:
            self.set_addr(addr)
            self.set_telephone(tel)
            self.vcard.set_revision()    

def _scale_image(i, widget):
    import math
    coords=widget.get_allocation()
    area = (coords[2]-coords[0])*(coords[3]-coords[0])/4
    w=i.rgb_width
    h=i.rgb_height
    i_area=math.sqrt(w*h)
    area = math.sqrt(area)
    w=(w*area)/i_area
    h=(h*area)/i_area
    return i.render(w, h)

class VCardView(gtk.GtkWindow):
    def __init__(self, vcard):
        gtk.GtkWindow.__init__(self)
        self.set_name("Contact")
        self.set_title("Contact")
        self.set_default_size(240,320)
        self.connect("destroy", self.OnDestroy)
        self.vcard = vcard

        gui=gtk.GtkScrolledWindow()
        text=gtk.GtkText()
        infobox=gtk.GtkVBox()
        text.set_word_wrap(1)
        text.set_line_wrap(1)
        self.text=text
        self.add_text("", str(vcard.get_name()))
        self.add_text("Home Address:", str(vcard.get_home_address()))
        self.add_text("Home Phone:", str(vcard.get_home_phone()))
        self.add_text("Work Address:", str(vcard.get_work_address()))
        self.add_text("Work Phone:", str(vcard.get_work_phone()))
        self.add_text("Mobile Phone:", str(vcard.get_mobile_phone()))
        self.add_text("Email:", "\n".join(map(str, vcard.get_email_addrs())))
        text.set_editable(0)
        infobox.pack_end(text)
        gui.add_with_viewport(infobox)

        buttons=HButtonBox(Button("Close", self.destroy))

        vbox=gtk.GtkVBox()
        vbox.pack_end(buttons, 0, 0)
        vbox.pack_end(gui)

        self.add(vbox)
        self.show_all()

        photo_url = vcard.get_photo_jpeg_url()
        print photo_url
        if photo_url:
            try:
                import urllib, GdkImlib
                tmpname, headers = urllib.urlretrieve(photo_url)
                print tmpname
                image = GdkImlib.Image(tmpname)
                _scale_image(image, self)
                pixmap=image.make_pixmap()
                infobox.pack_start(pixmap,0,0)
                pixmap.show()
            except:
                print "unable to load photo!", sys.exc_info()[0], sys.exc_info()[1]
 
    def add_text(self, title, item):
        if item and title: self.text.insert_defaults(title+"\n")
        if item: self.text.insert_defaults(item+"\n")
    def OnDestroy(self, widget):
        pass
        
class VCardEdit(gtk.GtkWindow):
    def __init__(self, vcard, mgr=None):
        gtk.GtkWindow.__init__(self)
        self.set_name("EditContact")
        self.vcard = vcard
        self.mgr = mgr
        self.panes = []

        self.book = gtk.GtkNotebook()
        self.panes.append(_VCard_ID(self))
        self.panes.append(_VCard_Photo(self))
        self.panes.append(_VCard_Addr(self, 0))
        self.panes.append(_VCard_Addr(self, 1))
        self.panes.append(_VCard_Mobile(self))
        self.panes.append(_VCard_Email(self))

        buttons=HButtonBox(Button("Save", self.OnSaveClicked, "Commit"),
                           Button("Cancel", self.destroy, "Discard"))

        vbox=gtk.GtkVBox()
        vbox.pack_start(self.book)
        vbox.pack_end(buttons, 0, 0)

        self.add(vbox)
        self.set_title("Edit Contact")
        self.set_default_size(240,320)
        self.connect("destroy", self.OnDestroy)
        self.show_all()

    def OnSaveClicked(self, widget):
        for p in self.panes: p.commit()
        if self.mgr: self.mgr.add_vcard(self.vcard)
        self.destroy()

    def OnDestroy(self, widget):
        pass

class VCardMgrView(gtk.GtkWindow):
    def __init__(self, mgr):
        gtk.GtkWindow.__init__(self)
        self.set_name("ContactMgr")
        self.mgr = mgr
        
        self.set_title("People")
        self.set_default_size(240,320)
        self.connect("destroy", self.OnDestroy)

##        try: self.pconfig = cPickle.load(open("/tmp/.people-config", "rb"))
##        except: self.pconfig = _ToDoMgrView_config()

        # - Menu bar
        menu = gtk.GtkMenuBar()
        file_item = MenuItem("People", None)
        file_menu = gtk.GtkMenu()
        quit_item = MenuItem("Quit", self.destroy)
        file_menu.add(quit_item)
        file_item.set_submenu(file_menu)
        menu.add(file_item)

        # - Button bar
        buttons = HButtonBox(Button("New", self.OnNewClicked, "Create"),
                             Button("Edit", self.OnEditClicked),
                             Button("View", self.OnViewClicked),
                             Button("Destroy", self.OnDestroyClicked, "Destroy"),
                             Button("Quit", self.destroy))

        # - Create the list
        self.list = gtk.GtkList()
        self.list.connect("select-child", self.OnVCardSelected)
        self.selected = None
        self.Refresh()

        # - Build the window layout
        vbox = gtk.GtkVBox()
        vbox.pack_start(menu, 0, 0)
        list_wnd = gtk.GtkScrolledWindow()
        list_wnd.add_with_viewport(self.list)
        list_wnd.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        vbox.add(list_wnd)
        vbox.pack_end(buttons, 0, 0)
        self.add(vbox)

        self.show_all()

    def OnDestroy(self, widget):
        
        self.mgr.dump_now()
##        try: cPickle.dump(self.pconfig, open("/tmp/.toad-config", "wb"))
##        except: print "warning: unable to save view mode"
        gtk.mainquit()

    # - The bottom Button Bar
    def OnNewClicked(self, widget):
        vcard = self.mgr.new_vcard()
        vcard_view = VCardEdit(vcard, self.mgr)
        vcard_view.connect("destroy", self.OnVCardEditClosed, vcard_view)
    def OnEditClicked(self, widget):
        if self.selected:
            vcard_view = VCardEdit(self.selected, self.mgr)
            vcard_view.connect("destroy", self.OnVCardEditClosed, vcard_view)
    def OnViewClicked(self, widget):
        if self.selected:
            vcard_view = VCardView(self.selected)
    def OnDestroyClicked(self, widget):
        if self.selected:
            self.mgr.rem_vcard(self.selected)
            self.selected = None
            self.Refresh()

    # - VCard item actions
    def OnVCardEditClosed(self, widget, view):
        self.selected = view.vcard
        self.Refresh(view.vcard)
    def OnVCardSelected(self, widget, child):
        self.selected = child.get_data("vcard")

    # - Redrawing the list
    def Refresh(self, vcard_refresh=None):
        vcards=self.mgr.get_sorted_vcards()
        dead=[]
        shown=[]

        # - Remove dead display entries
        for item in self.list.children()[:]:
            vcard = item.get_data("vcard")
            vcard_rev=item.get_data("vcard_rev")
            if ((vcard not in vcards) or
                (vcard.get_revision() != vcard_rev)):
                dead.append(item)
            else:
                shown.append(vcard)
        self.list.remove_items(dead)

        # - Insert new items (in the right place!)
        new_items = []
        select = None
        for vcard in vcards:
            if vcard not in shown:
                item = gtk.GtkListItem(str(vcard.get_name()))
                item.set_data("vcard", vcard)
                item.set_data("vcard_rev", vcard.get_revision())
                item.show()
                if vcard == self.selected: select = item
                self.list.insert_items([item], vcards.index(vcard))
        if select: select.select()
        self.mgr.dump()

def Main(argv):
    if len(argv) >= 2:
        filename = argv[1]
    else:
        filename = "/tmp/people.vcf"
    vcard_mgr=VCardMgr(filename)
    view = VCardMgrView(vcard_mgr)
    gtk.mainloop()

if __name__ == "__main__": Main(sys.argv)
